package yuehaoxitong

import (
	"shylinux.com/x/ice"
	"shylinux.com/x/icebergs/base/mdb"
	kit "shylinux.com/x/toolkits"

	"shylinux.com/x/community/src/yuehaoxitong/model"
)

type schedule struct {
	Table
	queue     queue
	userQueue userQueue
	reception reception
	volume    volume
	plan      string `name:"plan begin_time@date end_time@date"`
	expire    string `name:"expire" role:"void"`
	finish    string `name:"finish" role:"void"`
}

func (s schedule) Create(m *ice.Message, arg ...string) {
	defer m.ToastProcess("约号中...")("约号成功")
	s.Transaction(m, func() {
		msg := m.Cmd(s.volume, s.SelectForUpdate, model.UID, m.Option(model.VOLUME_UID))
		if m.Warn(kit.Int(msg.Append(model.COUNT)) >= kit.Int(msg.Append(model.AMOUNT)), "本服务已约满") {
			return
		}
		s.addCount(m, mdb.COUNT, "1")
		s.Table.Create(m, arg...)
	})
	s.sendTemplate(m, "约号成功", m.Result())
}
func (s schedule) Call(m *ice.Message, arg ...string) {
	defer m.ToastProcess("叫号中...")("叫号成功")
	if !s.checkRole(m, UserQueueCreator, UserQueueWorker) {
		return
	}
	s.Orders(m, model.TAKE_TIME).Limit(m, 1)
	s.SelectByStatus(m, m.Option(model.QUEUE_UID), kit.Format(ScheduleTake)).PushAction(s.Expire, s.Finish)
	// s.Select(m, kit.Simple(m.OptionSimple(model.QUEUE_UID, model.RECEPTION_UID), mdb.STATUS, ScheduleTake)...)
	if m.Length() == 0 {
		m.ProcessRewrite(model.UID, "")
		return
	}
	s.changeStatus(m, ScheduleTake, ScheduleCall, arg...)
	s.sendTemplate(m, "服务已叫号", m.Append(model.UID))
	m.ProcessRewrite(model.UID, m.Append(model.UID))
}
func (s schedule) List(m *ice.Message, arg ...string) {
	role := UserQueueRole(kit.Int(m.Cmd(s.userQueue, s.Select, m.OptionSimple(model.QUEUE_UID, model.USER_UID)).Append(model.ROLE)))
	s.Tables(m, s.volume).FieldsWithCreatedAT(m, s, model.SCHEDULE_STATUS, model.BEGIN_TIME, model.END_TIME,
		model.CANCEL_TIME, model.TAKE_TIME, model.CALL_TIME, model.EXPIRE_TIME, model.FINISH_TIME,
		s.Key(s, model.OPERATOR), s.Key(s, model.RECEPTION_UID), model.USER_UID, model.VOLUME_UID,
	)
	if role == UserQueueCreator || role == UserQueueManager {
		if len(arg) == 1 {
			s.Select(m, s.Key(s, model.QUEUE_UID), arg[0])
		} else if len(arg) == 2 {
			s.SelectDetail(m, s.Key(s, model.QUEUE_UID), arg[0], s.Key(s, model.UID), arg[1])
		}
		m.Table(func(value ice.Maps) {
			switch ScheduleStatus(kit.Int(value[model.SCHEDULE_STATUS])) {
			case SchedulePlan:
				m.PushButton(s.Cancel)
			case ScheduleCall:
				m.PushButton(s.Expire, s.Finish)
			default:
				m.PushButton()
			}
		})
	} else if role == UserQueueWorker {
		if len(arg) == 1 {
			s.Select(m, s.Key(s, model.QUEUE_UID), arg[0], s.Key(s, model.OPERATOR), m.Option(model.USER_UID))
		} else if len(arg) == 2 {
			s.SelectDetail(m, s.Key(s, model.QUEUE_UID), arg[0], s.Key(s, model.OPERATOR), m.Option(model.USER_UID), s.Key(s, model.UID), arg[1])
		}
		m.Table(func(value ice.Maps) {
			switch ScheduleStatus(kit.Int(value[model.SCHEDULE_STATUS])) {
			case ScheduleCall:
				m.PushButton(s.Expire, s.Finish)
			default:
				m.PushButton()
			}
		})
	} else {
		if len(arg) == 1 {
			s.Select(m, s.Key(s, model.QUEUE_UID), arg[0], model.USER_UID, m.Option(model.USER_UID))
		} else if len(arg) == 2 {
			s.SelectDetail(m, s.Key(s, model.QUEUE_UID), arg[0], model.USER_UID, m.Option(model.USER_UID), s.Key(s, model.UID), arg[1])
		}
		m.Table(func(value ice.Maps) {
			switch ScheduleStatus(kit.Int(value[model.SCHEDULE_STATUS])) {
			case SchedulePlan:
				m.PushButton(s.Cancel)
			default:
				m.PushButton()
			}
		})
	}
	s.SelectJoin(m, s.reception)
	s.SelectJoinUser(m)
	s.RewriteAppend(m)
	m.Action()
}
func (s schedule) SelectByStatus(m *ice.Message, arg ...string) *ice.Message {
	order := m.Option(mdb.ORDER)
	s.Tables(m, s.volume).FieldsWithCreatedAT(m, s, model.SCHEDULE_STATUS, model.BEGIN_TIME, model.END_TIME,
		model.CANCEL_TIME, model.TAKE_TIME, model.CALL_TIME, model.EXPIRE_TIME, model.FINISH_TIME,
		s.Key(s, model.OPERATOR), s.Key(s, model.RECEPTION_UID), model.USER_UID, model.VOLUME_UID,
	)
	kit.If(order, func() { s.Orders(m, order) })
	s.Select(m, kit.Simple(s.Key(s, model.QUEUE_UID), arg[0], model.STATUS, arg[1], arg[2:])...).Action()
	s.SelectJoin(m, s.reception)
	s.SelectJoinUser(m)
	s.RewriteAppend(m)
	return m
}
func (s schedule) Cancel(m *ice.Message, arg ...string) {
	defer m.ToastProcess("取消中...")("取消成功")
	if m.WarnNotFound(s.Select(m, m.OptionSimple(model.UID, model.USER_UID)...).Length() == 0) {
		return
	}
	s.Transaction(m, func() {
		s.addCount(m, mdb.COUNT, "-1")
		s.changeStatus(m, SchedulePlan, ScheduleCancel, arg...)
	})
	s.sendTemplate(m, "约号取消")
	m.ProcessRefresh()
}
func (s schedule) Take(m *ice.Message, arg ...string) {
	defer m.ToastProcess("取号中...")("取号成功")
	if m.WarnNotFound(s.Select(m, m.OptionSimple(model.UID, model.USER_UID)...).Length() == 0) {
		return
	}
	s.changeStatus(m, SchedulePlan, ScheduleTake, arg...)
	s.sendTemplate(m, "取号成功")
	m.ProcessRefresh()
}
func (s schedule) Expire(m *ice.Message, arg ...string) {
	defer m.ToastProcess("过号中...")("过号成功")
	if s.checkRole(m, UserQueueCreator, UserQueueWorker) {
		s.Transaction(m, func() {
			s.addCount(m, "", "1")
			s.changeStatus(m, ScheduleCall, ScheduleExpire, arg...)
		})
		s.Select(m.FieldsSetDetail(), model.UID, m.Option(model.UID))
		s.sendTemplate(m, "服务已过号")
		m.ProcessRefresh()
	}
}
func (s schedule) Finish(m *ice.Message, arg ...string) {
	defer m.ToastProcess("完成中...")("完成成功")
	if s.checkRole(m, UserQueueCreator, UserQueueWorker) {
		s.Transaction(m, func() {
			s.addCount(m, "", "1")
			s.changeStatus(m, ScheduleCall, ScheduleFinish, arg...)
		})
		s.Select(m.FieldsSetDetail(), model.UID, m.Option(model.UID))
		s.sendTemplate(m, "服务已完成")
		m.ProcessRefresh()
	}
}

func init() { ice.TeamCtxCmd(schedule{}) }

func (s schedule) addCount(m *ice.Message, key, count string) {
	m.Cmd(s.volume, s.AddCount, kit.Select(m.ActionKey(), key), count, m.Option(model.VOLUME_UID))
}
func (s schedule) changeStatus(m *ice.Message, from, to ScheduleStatus, arg ...string) *ice.Message {
	key := m.ActionKey() + "_time"
	s.Table.ChangeStatus(m, m.Option(model.UID), int(from), int(to), kit.TransArgValueTime(append(arg, key, m.Time()), key)...)
	return m
}
func (s schedule) sendTemplate(m *ice.Message, title string, arg ...string) *ice.Message {
	queue := m.Cmd(s.queue, s.Select, model.UID, m.Option(model.QUEUE_UID))
	reception := m.Cmd(s.reception, s.Select, model.UID, m.Option(model.RECEPTION_UID))
	s.Table.SendTemplate(m, "", m.Append(model.USER_UID), title, queue.Append(model.NAME)+" "+reception.Append(model.NAME), kit.Select("", arg, 0), m.Option(model.QUEUE_UID))
	return m
}

type ScheduleStatus int

const (
	SchedulePlan ScheduleStatus = iota
	ScheduleCancel
	ScheduleTake
	ScheduleCall
	ScheduleExpire
	ScheduleFinish
)

var ScheduleStatusList = map[ScheduleStatus]string{
	SchedulePlan:   "plan",
	ScheduleCancel: "cancel",
	ScheduleTake:   "take",
	ScheduleCall:   "call",
	ScheduleExpire: "expire",
	ScheduleFinish: "finish",
}

func (s ScheduleStatus) String() string { return ScheduleStatusList[s] }
